(function (QUnit,sinon,videojs) {
'use strict';

QUnit = QUnit && QUnit.hasOwnProperty('default') ? QUnit['default'] : QUnit;
sinon = sinon && sinon.hasOwnProperty('default') ? sinon['default'] : sinon;
videojs = videojs && videojs.hasOwnProperty('default') ? videojs['default'] : videojs;

var commonjsGlobal = typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

var empty = {};


var empty$1 = (Object.freeze || Object)({
	'default': empty
});

var minDoc = ( empty$1 && empty ) || empty$1;

var topLevel = typeof commonjsGlobal !== 'undefined' ? commonjsGlobal :
    typeof window !== 'undefined' ? window : {};


var doccy;

if (typeof document !== 'undefined') {
    doccy = document;
} else {
    doccy = topLevel['__GLOBAL_DOCUMENT_CACHE@4'];

    if (!doccy) {
        doccy = topLevel['__GLOBAL_DOCUMENT_CACHE@4'] = minDoc;
    }
}

var document_1 = doccy;

function spriteThumbs(player, options) {
  var path = options.path;
  var height = options.height;
  var width = options.width;
  var interval = options.interval;

  if (!path.length || !height || !width) {
    return;
  }

  var dom = videojs.dom || videojs;
  var controls = player.controlBar;
  var progress = controls.progressControl;
  var seekBar = progress.seekBar;
  var mouseTimeDisplay = seekBar.mouseTimeDisplay;
  if (!mouseTimeDisplay) {
    return;
  }

  // 设置样式
  var tooltipStyle = function tooltipStyle(obj) {
    Object.keys(obj).forEach(function (key) {
      var val = obj[key];
      var ttStyle = mouseTimeDisplay.timeTooltip.el_.style;

      if (val !== '') {
        ttStyle.setProperty(key, val);
      } else {
        ttStyle.removeProperty(key);
      }
    });
  };

  var rows = 0;
  var columns = 0; // 缩略图列数
  var imgWidth = 0;
  var imgHeight = 0;
  var shots = 0; // 每张雪碧图有多少个截屏
  // load sprite early
  dom.createEl('img', {
    src: path[0]
  }).onload = function (ev) {
    var target = ev.target;

    imgWidth = target.naturalWidth;
    imgHeight = target.naturalHeight;
    columns = Math.floor(imgWidth / width);
    rows = Math.floor(imgHeight / height);
    shots = rows * columns;
  };

  var hijackMouseTooltip = function hijackMouseTooltip() {
    if (!columns) {
      return;
    }

    // 鼠标在进度条对应的时间
    var hoverTime = Math.floor(player.duration() * (parseFloat(mouseTimeDisplay.el_.style.left) / seekBar.el_.clientWidth));
    if (isNaN(hoverTime)) {
      return;
    }

    var currentShot = hoverTime % (interval * shots); // 第几张截屏
    var row = Math.floor(currentShot % (interval * columns)) / interval;
    var column = Math.floor(currentShot / (interval * columns));
    var cLeft = -row * width;
    var cTop = -column * height;
    var controlsTop = dom.getBoundingClientRect(controls.el_).top;
    var seekBarTop = dom.getBoundingClientRect(seekBar.el_).top;
    // top of seekBar is 0 position
    var topOffset = -height;
    if (controlsTop < seekBarTop) {
      topOffset -= seekBarTop - controlsTop;
    }

    var page = 0;
    if (path.length > 1) {
      page = Math.floor(hoverTime / interval / shots);
    }

    tooltipStyle({
      'width': width + 'px',
      'height': height + 'px',
      'background-image': 'url(' + path[page] + ')',
      'background-repeat': 'no-repeat',
      'background-position': cLeft + 'px ' + cTop + 'px',
      'top': topOffset + 'px',
      'color': '#fff',
      'text-shadow': '1px 1px #000',
      'border': '1px solid #000',
      'margin': '0 1px'
    });
  };

  player.ready(function () {
    progress.on('mousemove', hijackMouseTooltip);
    progress.on('touchmove', hijackMouseTooltip);
  });
  player.addClass('vjs-sprite-thumbnails');
}

var version = "0.0.1";

/**
 * Default plugin options
 *
 * @param {String} path
 *        Sprite location. Must be set by user.
 * @param {Integer} width
 *        Width in pixels of a thumbnail. Must be set by user.
 * @param {Integer} height
 *        Height in pixels of a thumbnail. Must be set by user.
 * @param {Number} interval
 *        Interval between thumbnail frames in seconds. Default: 1.
 */
var defaults = {
  path: [],
  width: 0,
  height: 0,
  interval: 1
};

/**
 * The video.js sprite thumbnails plugin.
 *
 * Invokes spriteThumbs to set up and display thumbnails from a sprite image
 * when the user hovers over the progress bar.
 *
 * @function spriteThumbnails
 * @param    {Object} options
 *           Object accepting 4 plugin configuration parameters.
 */
var spriteThumbnails = function spriteThumbnails(options) {
  spriteThumbs(this, videojs.mergeOptions(defaults, options));
};

// Register the plugin with video.js.
videojs.registerPlugin('spriteThumbnails', spriteThumbnails);

// Include the version number.
spriteThumbnails.VERSION = version;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};

var Player = videojs.getComponent('Player');

QUnit.test('the environment is sane', function (assert) {
  assert.strictEqual(_typeof(Array.isArray), 'function', 'es5 exists');
  assert.strictEqual(typeof sinon === 'undefined' ? 'undefined' : _typeof(sinon), 'object', 'sinon exists');
  assert.strictEqual(typeof videojs === 'undefined' ? 'undefined' : _typeof(videojs), 'function', 'videojs exists');
  assert.strictEqual(typeof spriteThumbnails === 'undefined' ? 'undefined' : _typeof(spriteThumbnails), 'function', 'plugin is a function');
});

QUnit.module('videojs-sprite-thumbnails', {
  beforeEach: function beforeEach() {

    // Mock the environment's timers because certain things - particularly
    // player readiness - are asynchronous in video.js 5. This MUST come
    // before any player is created; otherwise, timers could get created
    // with the actual timer methods!
    this.clock = sinon.useFakeTimers();

    this.fixture = document_1.getElementById('qunit-fixture');
    this.video = document_1.createElement('video');
    this.fixture.appendChild(this.video);
    this.player = videojs(this.video);
  },
  afterEach: function afterEach() {
    this.player.dispose();
    this.clock.restore();
  }
});

QUnit.test('registers itself with video.js', function (assert) {
  assert.expect(2);

  assert.strictEqual(_typeof(Player.prototype.spriteThumbnails), 'function', 'videojs-sprite-thumbnails plugin was registered');

  this.player.spriteThumbnails({
    url: 'https://raw.githubusercontent.com/blacktrash/videojs-sprite-thumbnails/master/img/oceans-thumbs.jpg',
    width: 240,
    height: 100
  });

  // Tick the clock forward enough to trigger the player to be "ready".
  this.clock.tick(1);

  if (this.player.controlBar.progressControl.seekBar.mouseTimeDisplay !== undefined) {
    assert.ok(this.player.hasClass('vjs-sprite-thumbnails'), 'the plugin adds a class to the player');
  } else {
    assert.ok(!this.player.hasClass('vjs-sprite-thumbnails'), 'no mouseTimeDisplay: no-op plugin does not advertise itself');
  }
});

}(QUnit,sinon,videojs));
